==> FreeDoko_0.7.8.src.zip: FreeDoko_0.7.8/src/player/rating/second_min.h <==
/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001-2010  by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@gmx.de
 *    Borg Enders  borg@borgsoft.de
 *
 *********************************************************************/

#ifndef CLASS_RATING_SECOND_MIN_HEADER
#define CLASS_RATING_SECOND_MIN_HEADER

#include "rating.h"

/**
 ** second minimum rater
 ** returns the second minimal value of the given values
 **/
class Rating::SecondMin : public Rating
{
  public:
    // constructor
    SecondMin() : Rating(SECOND_MIN),
		  min_value(INT_MAX), second_min_value(INT_MAX)
    { }
    // destructor
    ~SecondMin() { }

    // adds a value
    void add(int const value)
    {
      if (value > this->second_min_value) {
	if (value > this->min_value) {
	  this->second_min_value = this->min_value;
	  this->min_value = value;
	} else {
	  this->second_min_value = value;
	}
      } // if (value > this->second_min_value)
    } // void add(int const value)

    // returns the rated value
    int value() const
    {
      if (this->second_min_value == INT_MAX)
	return this->min_value;
      return this->second_min_value;
    } // int rating() const

  private:
    // the minimal value
    int min_value;
    // the second minimal value
    int second_min_value;
}; // class Rating::SecondMin : public Rating

#endif // #ifndef CLASS_RATING_SECOND_MIN_HEADER

==> FreeDoko_0.7.11.src.zip: FreeDoko_0.7.11/src/player/rating/second_min.h <==
/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001-2011  by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@gmx.de
 *    Borg Enders  borg@borgsoft.de
 *
 *********************************************************************/

#ifndef CLASS_RATING_SECOND_MIN_HEADER
#define CLASS_RATING_SECOND_MIN_HEADER

#include "rating.h"

/**
 ** second minimum rater
 ** returns the second minimal value of the given values
 **/
class Rating::SecondMin : public Rating
{
  public:
    // constructor
    SecondMin() : Rating(SECOND_MIN) { }
    // destructor
    ~SecondMin() { }

    // returns the rated value
    int value() const
    { 
      if (this->values.empty())
        return INT_MAX;
      if (this->values.size() == 1)
        return *this->values.begin();
      this->sort();
      return *(this->values.begin() + 1);
    }

  private:
}; // class Rating::SecondMin : public Rating

#endif // #ifndef CLASS_RATING_SECOND_MIN_HEADER
